@@ -6,13 +6,13 @@ This module provides classes to define electronic structure, such as the density
 
 from __future__ import division
 
-__author__="Shyue Ping Ong"
+__author__="Shyue Ping Ong, Vincent L Chevrier"
 __copyright__ = "Copyright 2011, The Materials Project"
 __version__ = "1.0"
 __maintainer__ = "Shyue Ping Ong"
 __email__ = "shyue@mit.edu"
 __status__ = "Production"
-__date__ ="$Sep 23, 2011M$"
+__date__ ="Sep 23, 2011"
 
 import numpy as np
 import scipy.interpolate as spint
