@@ -13,7 +13,7 @@ __version__ = "1.0"
 __maintainer__ = "Shyue Ping Ong"
 __email__ = "shyue@mit.edu"
 __status__ = "Production"
-__date__ ="$Sep 23, 2011M$"
+__date__ ="Sep 23, 2011"
 
 import re
 import math
@@ -363,18 +363,18 @@ class PeriodicSite(Site):
         if jimage == None:
             adj1 = np.array([- math.floor(i) for i in self._fcoords])
             adj2 = np.array([- math.floor(i) for i in fcoords])
-            mindist = 1e100
+            mindist = float('inf')
             coord1 = self._fcoords + adj1
             coord2 = fcoords + adj2
             test_set = [[-1,0] if coord1[i] < coord2[i] else [0,1] for i in xrange(3)]
             for image in itertools.product(*test_set):
-                dist = np.linalg.norm(self.lattice.get_cartesian_coords(coord2 + image - coord1))
+                dist = np.linalg.norm(self._lattice.get_cartesian_coords(coord2 + image - coord1))
                 if dist < mindist:
                     mindist = dist
                     jimage = adj2 - adj1 + image
             return mindist, jimage        
         return np.linalg.norm(self.lattice.get_cartesian_coords(jimage + fcoords - self._fcoords)), jimage
-    
+
     def distance_and_image(self, other, jimage=None):
         """
         Gets distance and instance between two sites assuming periodic boundary conditions.
@@ -631,8 +631,7 @@ class Structure(collections.Sequence, collections.Hashable):
                     neighbors.append((PeriodicSite(self._sites[i].species_and_occu, fcoords[i], self._lattice), dists[i]))
         
         return neighbors
-    
-    
+       
     def get_neighbors(self, site, r):
         """
         Get all neighbors to a site within a sphere of radius r.  Excludes the site itself.
@@ -1196,4 +1195,4 @@ class Composition (collections.Mapping, collections.Hashable):
 
 if __name__ == "__main__":
     import doctest
-    doctest.testmod()
+    doctest.testmod()
\ No newline at end of file
